@@ -122,7 +122,10 @@ exchange_folder_get_message_data (CamelFolder *folder,
 	if (stream) {
 		CamelStream *null_stream = camel_stream_null_new ();
 
-		camel_stream_reset (stream, NULL);
+		g_seekable_seek (
+			G_SEEKABLE (stream), 0,
+			G_SEEK_SET, NULL, NULL);
+
 		if (camel_stream_write_to_stream (stream, null_stream, cancellable, NULL) <= 0) {
 			stream = NULL;
 		}
@@ -134,10 +137,14 @@ exchange_folder_get_message_data (CamelFolder *folder,
 		ba = g_byte_array_new ();
 		stream_mem = camel_stream_mem_new ();
 		camel_stream_mem_set_byte_array (CAMEL_STREAM_MEM (stream_mem), ba);
-		camel_stream_reset (stream, NULL);
+
+		g_seekable_seek (
+			G_SEEKABLE (stream), 0,
+			G_SEEK_SET, NULL, NULL);
+
 		camel_stream_write_to_stream (stream, stream_mem, cancellable, NULL);
-		g_object_unref (CAMEL_OBJECT (stream_mem));
-		g_object_unref (CAMEL_OBJECT (stream));
+		g_object_unref (stream_mem);
+		g_object_unref (stream);
 
 		return ba;
 	}
@@ -246,7 +253,7 @@ exchange_folder_transfer_messages_the_hard_way (CamelFolder *source,
 		*transferred_uids = g_ptr_array_new ();
 
 	for (i = 0; i < uids->len; i++) {
-		info = camel_folder_summary_uid (source->summary, uids->pdata[i]);
+		info = camel_folder_summary_get (source->summary, uids->pdata[i]);
 		if (!info)
 			continue;
 
@@ -441,11 +448,11 @@ exchange_folder_append_message_sync (CamelFolder *folder,
 	store = CAMEL_EXCHANGE_STORE (parent_store);
 
 	/*
-	   FIXME: We should add a top-level camel API camel_mime_message_prune_invalid_chars
-	   which each of the provider will have to implement to remove things
-	   that are invalid for their Transport mechanism. This will help in
-	   avoiding  duplication of work. Now Sending and Importing both requires
-	   substitution of \t and \n with blank.
+	 * FIXME: We should add a top-level camel API camel_mime_message_prune_invalid_chars
+	 * which each of the provider will have to implement to remove things
+	 * that are invalid for their Transport mechanism. This will help in
+	 * avoiding  duplication of work. Now Sending and Importing both requires
+	 * substitution of \t and \n with blank.
 	*/
 
 	old_subject = g_strdup (camel_mime_message_get_subject (message));
@@ -638,8 +645,9 @@ exchange_folder_refresh_info_sync (CamelFolder *folder,
 	success = success && (!error || !*error);
 
 	if (success) {
-		folder->summary->unread_count = unread_count;
-		folder->summary->visible_count = visible_count;
+		/* do nothing, the summary should be in sync on its own */
+		/* folder->summary->unread_count = unread_count;
+		folder->summary->visible_count = visible_count; */
 	} else if (error && !*error) {
 		g_set_error (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC, "%s", _("Could not get new messages"));
 	}
@@ -656,7 +664,7 @@ exchange_folder_synchronize_sync (CamelFolder *folder,
 	if (expunge)
 		exchange_folder_expunge_sync (folder, cancellable, NULL);
 
-	return camel_folder_summary_save_to_db (folder->summary, error) == 0;
+	return camel_folder_summary_save_to_db (folder->summary, error);
 }
 
 static gboolean
@@ -701,7 +709,7 @@ exchange_folder_transfer_messages_to_sync (CamelFolder *source,
 		journal = (CamelExchangeJournal *) exch_dest->journal;
 
 		for (i = 0; i < uids->len && success; i++) {
-			info = camel_folder_summary_uid (
+			info = camel_folder_summary_get (
 				source->summary, uids->pdata[i]);
 			if (!info)
 				continue;
@@ -804,7 +812,8 @@ camel_exchange_folder_init (CamelExchangeFolder *exchange_folder)
  * base64 representation of this value.
  */
 static CamelSummaryMessageID *
-find_parent (CamelExchangeFolder *exch, const gchar *thread_index)
+find_parent (CamelExchangeFolder *exch,
+             const gchar *thread_index)
 {
 	CamelSummaryMessageID *msgid;
 	guchar *decoded;
@@ -840,9 +849,11 @@ find_parent (CamelExchangeFolder *exch, const gchar *thread_index)
  **/
 void
 camel_exchange_folder_add_message (CamelExchangeFolder *exch,
-				   const gchar *uid, guint32 flags,
-				   guint32 size, const gchar *headers,
-				   const gchar *href)
+                                   const gchar *uid,
+                                   guint32 flags,
+                                   guint32 size,
+                                   const gchar *headers,
+                                   const gchar *href)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfo *info;
@@ -851,7 +862,7 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	CamelStream *stream;
 	CamelMimeMessage *msg;
 
-	info = camel_folder_summary_uid (folder->summary, uid);
+	info = camel_folder_summary_get (folder->summary, uid);
 	if (info) {
 		camel_message_info_free (info);
 		return;
@@ -890,11 +901,6 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 
 	camel_folder_summary_add (folder->summary, info);
 
-	if (!(flags & CAMEL_MESSAGE_SEEN)) {
-		folder->summary->unread_count++;
-		folder->summary->visible_count++;
-	}
-
 	changes = camel_folder_change_info_new ();
 	camel_folder_change_info_add_uid (changes, uid);
 	camel_folder_change_info_recent_uid (changes, uid);
@@ -912,14 +918,14 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
  **/
 void
 camel_exchange_folder_remove_message (CamelExchangeFolder *exch,
-				      const gchar *uid)
+                                      const gchar *uid)
 {
 	CamelFolderSummary *summary = CAMEL_FOLDER (exch)->summary;
 	CamelFolderChangeInfo *changes;
 	CamelMessageInfo *info;
 	CamelExchangeMessageInfo *einfo;
 
-	info = camel_folder_summary_uid (summary, uid);
+	info = camel_folder_summary_get (summary, uid);
 	if (!info)
 		return;
 
@@ -954,7 +960,7 @@ camel_exchange_folder_remove_message (CamelExchangeFolder *exch,
  **/
 void
 camel_exchange_folder_uncache_message (CamelExchangeFolder *exch,
-				       const gchar *uid)
+                                       const gchar *uid)
 {
 	camel_data_cache_remove (exch->cache, "cache", uid, NULL);
 }
@@ -970,28 +976,23 @@ camel_exchange_folder_uncache_message (CamelExchangeFolder *exch,
  **/
 void
 camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
-					    const gchar *uid, guint32 flags)
+                                            const gchar *uid,
+                                            guint32 flags)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfoBase *info;
-	CamelFolderChangeInfo *changes;
 
-	info = (CamelMessageInfoBase *) camel_folder_summary_uid (folder->summary, uid);
+	info = (CamelMessageInfoBase *) camel_folder_summary_get (folder->summary, uid);
 	if (!info)
 		return;
 
 	flags |= (info->flags & ~CAMEL_EXCHANGE_SERVER_FLAGS);
 
 	if (info->flags != flags) {
-		info->flags = flags;
-		info->dirty = 1;
-		camel_folder_summary_touch (folder->summary);
-
-		changes = camel_folder_change_info_new ();
-		camel_folder_change_info_change_uid (changes, uid);
-		camel_folder_changed (CAMEL_FOLDER (exch), changes);
-		camel_folder_change_info_free (changes);
+		camel_message_info_set_flags ((CamelMessageInfo *) info, ~0, flags);
 	}
+
+	camel_message_info_free ((CamelMessageInfo *) info);
 }
 
 /**
@@ -1006,33 +1007,28 @@ camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
  **/
 void
 camel_exchange_folder_update_message_flags_ex (CamelExchangeFolder *exch,
-					       const gchar *uid, guint32 flags,
-					       guint32 mask)
+                                               const gchar *uid,
+                                               guint32 flags,
+                                               guint32 mask)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfoBase *info;
-	CamelFolderChangeInfo *changes;
 
-	info = (CamelMessageInfoBase *) camel_folder_summary_uid (folder->summary, uid);
+	info = (CamelMessageInfoBase *) camel_folder_summary_get (folder->summary, uid);
 	if (!info)
 		return;
 
 	mask &= CAMEL_EXCHANGE_SERVER_FLAGS;
 	if (!mask) {
+		camel_message_info_free ((CamelMessageInfo *) info);
 		return;
 	}
 
 	if ((info->flags & mask) != (flags & mask)) {
-		info->flags &= ~mask;
-		info->flags |= (flags & mask);
-		info->dirty = 1;
-		camel_folder_summary_touch (folder->summary);
-
-		changes = camel_folder_change_info_new ();
-		camel_folder_change_info_change_uid (changes, uid);
-		camel_folder_changed (CAMEL_FOLDER (exch), changes);
-		camel_folder_change_info_free (changes);
+		camel_message_info_set_flags ((CamelMessageInfo *) info, mask, flags);
 	}
+
+	camel_message_info_free ((CamelMessageInfo *) info);
 }
 
 /**
@@ -1047,15 +1043,15 @@ camel_exchange_folder_update_message_flags_ex (CamelExchangeFolder *exch,
  **/
 void
 camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
-					  const gchar *uid,
-					  const gchar *name,
-					  const gchar *value)
+                                          const gchar *uid,
+                                          const gchar *name,
+                                          const gchar *value)
 {
 	CamelFolder *folder = CAMEL_FOLDER (exch);
 	CamelMessageInfoBase *info;
 	CamelFolderChangeInfo *changes;
 
-	info = (CamelMessageInfoBase *) camel_folder_summary_uid (folder->summary, uid);
+	info = (CamelMessageInfoBase *) camel_folder_summary_get (folder->summary, uid);
 	if (!info)
 		return;
 
@@ -1100,6 +1096,7 @@ camel_exchange_folder_construct (CamelFolder *folder,
 	CamelStore *parent_store;
 	const gchar *full_name;
 	gint i, len = 0;
+	GPtrArray *known_uids;
 
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
@@ -1149,9 +1146,9 @@ camel_exchange_folder_construct (CamelFolder *folder,
 	exch->thread_index_to_message_id =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	len = camel_folder_summary_count (folder->summary);
-	for (i = 0; i < len; i++) {
-		info = camel_folder_summary_index (folder->summary, i);
+	known_uids = camel_folder_summary_get_array (folder->summary);
+	for (i = 0; known_uids && i < known_uids->len; i++) {
+		info = camel_folder_summary_get (folder->summary, g_ptr_array_index (known_uids, i));
 		einfo = (CamelExchangeMessageInfo *) info;
 
 		if (einfo->thread_index && einfo->info.message_id.id.id) {
@@ -1163,6 +1160,8 @@ camel_exchange_folder_construct (CamelFolder *folder,
 		camel_message_info_free (info);
 	}
 
+	camel_folder_summary_free_array (known_uids);
+
 	if (parent_store != NULL) {
 		gboolean ok, create = camel_flags & CAMEL_STORE_FOLDER_CREATE, readonly = FALSE;
 
@@ -1178,7 +1177,7 @@ camel_exchange_folder_construct (CamelFolder *folder,
 
 		for (i = 0; i < summary->len; i++) {
 			uids->pdata[i] = summary->pdata[i];
-			info = camel_folder_summary_uid (folder->summary, uids->pdata[i]);
+			info = camel_folder_summary_get (folder->summary, uids->pdata[i]);
 			flags->data[i] = ((CamelMessageInfoBase *) info)->flags & CAMEL_EXCHANGE_SERVER_FLAGS;
 			hrefs->pdata[i] = ((CamelExchangeMessageInfo *) info)->href;
 			//camel_tag_list_free (&((CamelMessageInfoBase *) info)->user_tags);
